// ------------------------------------------------------------------------------------
//
// Copyright  2006 Intel Corporation
// All Rights Reserved
// 
// The sample source code contained or described herein and all documents
// related to the sample source code ("Material") are owned by Intel Corporation
// or its suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material may contain trade secrets and
// proprietary and confidential information of Intel Corporation and its suppliers
// and licensors, and is protected by worldwide copyright and trade secret laws and
// treaty provisions. The sample source code is provided AS IS, with no warranties
// of any kind, express or implied. Except as expressly permitted by the Software
// license, neither Intel Corporation nor its suppliers assumes any responsibility
// or liability for any errors or inaccuracies that may appear herein.
//
// ------------------------------------------------------------------------------------
#ifndef CRITICALSECTION_H
#define CRITICALSECTION_H

#include "stdafx.h"

#ifdef WIN32
#include <process.h>
#elif LINUX
#include <pthread.h>
#include <unistd.h>
#endif

/** \brief protects a section of code from simultaneous access by multiple threads

CriticalSection objects are used to isolate a section of code such that only one thread can access it at a time.
Once enter() is called on a CriticalSection object, other threads will block on enter() until leave() is called
on that object.
*/
class CriticalSection
{
public:
    CriticalSection(void);
    ~CriticalSection(void);
    
    BOOL isValid(void);
    BOOL enter(void);
    BOOL leave(void);

private:
    #ifdef WIN32
        typedef CRITICAL_SECTION CriticalSectionType;
    #elif LINUX
        typedef pthread_mutex_t *CriticalSectionType;
    #endif
    
    CriticalSectionType m_criticalSection;
    BOOL m_valid;
};

#endif
