// ------------------------------------------------------------------------------------
//
// Copyright  2006 Intel Corporation
// All Rights Reserved
// 
// The sample source code contained or described herein and all documents
// related to the sample source code ("Material") are owned by Intel Corporation
// or its suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material may contain trade secrets and
// proprietary and confidential information of Intel Corporation and its suppliers
// and licensors, and is protected by worldwide copyright and trade secret laws and
// treaty provisions. The sample source code is provided AS IS, with no warranties
// of any kind, express or implied. Except as expressly permitted by the Software
// license, neither Intel Corporation nor its suppliers assumes any responsibility
// or liability for any errors or inaccuracies that may appear herein.
//
// ------------------------------------------------------------------------------------
#ifndef OPENGLWINDOW_H
#define OPENGLWINDOW_H

#include "stdafx.h"
#include "ThreadManager.h"
#include "CriticalSection.h"
#include <SDL.h>

/** \brief creates and manages an OpenGL window

OpenGLWindow provides a simple way to make a multi-threaded event loop with OpenGL rendering using SDL.
Your event thread should call runWindowLoop() as its main event loop, and your rendering thread should call
enableRender() before making any OpenGL calls, and whenever the value returned by getWindowID() changes.
OpenGLWindow only supports one thread rendering at a time; when a thread is ready to relinquish rendering
duties, it should call disableRender().
*/
class OpenGLWindow
{
public:
    // type definition for functions passed to runWindowLoop()
    typedef BOOL (*CallbackLoopFunction)(void);
    typedef void (*CallbackKeypressFunction)(SDL_KeyboardEvent *);

    OpenGLWindow(void);
    ~OpenGLWindow(void);
    
    void runWindowLoop(
        const char *windowName,
        int width,
        int height,
        CallbackLoopFunction loopFunc = NULL,
        CallbackKeypressFunction keypressFunc = NULL
    );
    
    int getWindowID(void);
    
    BOOL isFullscreen(void);
    void setFullscreen(BOOL fullscreen);
    
    BOOL enableRender(void);
    void disableRender(void);
    
    void update(void);
    
private:
    int m_width;
    int m_height;
    int m_windowID;
    int m_lastWindowID;
    char m_windowName[64];
    BOOL m_fullscreen;
    BOOL m_respawnWindow;
};

#endif
