// ------------------------------------------------------------------------------------
//
// Copyright  2006 Intel Corporation
// All Rights Reserved
// 
// The sample source code contained or described herein and all documents
// related to the sample source code ("Material") are owned by Intel Corporation
// or its suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material may contain trade secrets and
// proprietary and confidential information of Intel Corporation and its suppliers
// and licensors, and is protected by worldwide copyright and trade secret laws and
// treaty provisions. The sample source code is provided AS IS, with no warranties
// of any kind, express or implied. Except as expressly permitted by the Software
// license, neither Intel Corporation nor its suppliers assumes any responsibility
// or liability for any errors or inaccuracies that may appear herein.
//
// ------------------------------------------------------------------------------------
#ifndef THREADMANAGERSERIAL_H
#define THREADMANAGERSERIAL_H

#include "stdafx.h"
#include "ThreadManager.h"

/** \brief creates and manages a serializable thread pool

ThreadManagerSerial extends ThreadManager to allow the work functions assigned to threads in the pool to
instead be assigned to the primary thread.  In order to ensure proper behavior, your primary thread must
call doSerialWork() in its event loop.
*/
class ThreadManagerSerial: public ThreadManager
{
public:
    // thread-safe utility functions
    static ThreadManagerSerial &getThreadManagerSerial(void);

    // non thread-safe functions (call only from primary thread)
    void doSerialWork();
    BOOL unassignThread(int id);
    BOOL startThread(int id);
    BOOL stopThread(int id);
    BOOL setThreadSerialized(int id, BOOL serial);

    // thread-safe functions
    int getAndResetThreadCalls(int id);
    BOOL isThreadSerialized(int id);

protected:
    // constants for thread status
    enum {
        THREAD_STATUS_SERIAL_STOPPED                        = THREAD_STATUS_MAX << 1,
        THREAD_STATUS_SERIAL_RUNNING                        = THREAD_STATUS_MAX << 2,
        THREAD_STATUS_SERIAL_TO_PARALLEL_STOPPED_REQUEST    = THREAD_STATUS_MAX << 3,
        THREAD_STATUS_SERIAL_TO_PARALLEL_RUNNING_REQUEST    = THREAD_STATUS_MAX << 4,
        THREAD_STATUS_PARALLEL_TO_SERIAL_STOPPED_REQUEST    = THREAD_STATUS_MAX << 5,
        THREAD_STATUS_PARALLEL_TO_SERIAL_RUNNING_REQUEST    = THREAD_STATUS_MAX << 6,
        THREAD_STATUS_SERIAL_MAX                            = THREAD_STATUS_MAX << 6
    };

    ThreadManagerSerial(void);
    ~ThreadManagerSerial(void);
    
    void callbackHandleStatus(int id, int status);
    
    long m_serialCalls;
};

#endif
