// ------------------------------------------------------------------------------------
//
// Copyright  2006 Intel Corporation
// All Rights Reserved
// 
// The sample source code contained or described herein and all documents
// related to the sample source code ("Material") are owned by Intel Corporation
// or its suppliers or licensors. Title to the Material remains with Intel Corporation
// or its suppliers and licensors. The Material may contain trade secrets and
// proprietary and confidential information of Intel Corporation and its suppliers
// and licensors, and is protected by worldwide copyright and trade secret laws and
// treaty provisions. The sample source code is provided AS IS, with no warranties
// of any kind, express or implied. Except as expressly permitted by the Software
// license, neither Intel Corporation nor its suppliers assumes any responsibility
// or liability for any errors or inaccuracies that may appear herein.
//
// ------------------------------------------------------------------------------------
#ifndef WORLD_H
#define WORLD_H

#include "stdafx.h"
class OpenGLWindow;     // see OpenGLWindow.h for details
#include "CriticalSection.h"
#include <SDL.h>

#define USE_THREADS

class World
{
public:
    World(void);
    ~World(void);
    
    void init(int threadUpdateID, int threadRenderID, OpenGLWindow *window, int width, int height);
    void shutdown(void);
    
    void initOpenGL(void);
    void shutdownOpenGL(void);
    
    BOOL isPaused();
    void setPaused(BOOL paused);
    
    void update(void);
    void render(void);
    void handleKeypress(SDL_KeyboardEvent *event);
    
    BOOL isTimeToQuit(void);
    
private:
    static const int BACKGROUND_MAX = 300000;
    static const int BACKGROUND_STEP = 10000;
    static const int FOREGROUND_MAX = 1500;
    static const int FOREGROUND_STEP = 50;
    
    int m_threadUpdateID;
    int m_threadRenderID;
    OpenGLWindow *m_window;
    CriticalSection m_callsPerSecondCS;
    CriticalSection m_timeUpdateCS;
    BOOL m_paused;
    
    // data for the background
    struct BackgroundElement
    {
        GLfloat vertex[3][3];
        GLfloat color[3];
    };
    BackgroundElement m_background[BACKGROUND_MAX];
    int m_backgroundCount;
    int m_backgroundMax;
    
    // data for the foreground
    struct ForegroundElement
    {
        float radius;
        float mass;
        float rotation;
        float rotationVelocity;
        GLfloat location[3];
        GLfloat velocity[2];  // no velocity in z
        GLfloat color[3];
    };
    ForegroundElement m_foreground[FOREGROUND_MAX];
    int m_foregroundCount;
    int m_foregroundMax;
    int m_foregroundElementCallList;
    GLUquadricObj *m_foregroundElementQuadricObj;
    float m_foregroundElementRadius;
    
    // data for the label
    int m_millisecondsPassed;
    int m_eventCPS;
    int m_updateCPS;
    int m_renderCPS;
    char m_instructionBuffer[128];
    char m_instructionBuffer2[128];
    char m_cpsBuffer[128];
    
    int m_width;
    int m_height;
    float m_aspect;

    BOOL m_timeToQuit;

    float m_extentXFactor;
    float m_extentYFactor;
    float m_maxRadius;
    float m_foregroundRadius;
    float m_blackHoleRadius;
    float m_blackHoleMass;
    enum {
        BLACK_HOLE_STABLE,
        BLACK_HOLE_EXPLODE
    } m_blackHoleState;
    
    int m_font;
    
    void resize(int width, int height);
    
    BOOL areThreadsSerialized();
    void toggleThreadsSerialized();
    
    void toggleFullscreen();
    
    // background functions
    void adjustBackgroundCount(int delta);
    void generateBackground(void);
    void renderBackground(void);
    
    // foreground functions
    void adjustForegroundCount(int delta);
    void generateForeground(void);
    void renderForeground(void);
    void resetForegroundElement(ForegroundElement *e);
    float radiusForMass(float mass);
    void swapForegroundElement(ForegroundElement *e1, ForegroundElement *e2);
    
    // label functions
    int calculateTimeDeltaMilliseconds(void);
    void updateCallsPerSecond(int milliseconds);
    void renderLabel(void);
};

#endif
